@@ -26,12 +26,15 @@
 #include "dnsparser.hh"
 #include "dnsdist-cache.hh"
 #include "dnsdist-ecs.hh"
-#include "ednsoptions.hh"
 #include "ednssubnet.hh"
 #include "packetcache.hh"
 
 DNSDistPacketCache::DNSDistPacketCache(size_t maxEntries, uint32_t maxTTL, uint32_t minTTL, uint32_t tempFailureTTL, uint32_t maxNegativeTTL, uint32_t staleTTL, bool dontAge, uint32_t shards, bool deferrableInsertLock, bool parseECS): d_maxEntries(maxEntries), d_shardCount(shards), d_maxTTL(maxTTL), d_tempFailureTTL(tempFailureTTL), d_maxNegativeTTL(maxNegativeTTL), d_minTTL(minTTL), d_staleTTL(staleTTL), d_dontAge(dontAge), d_deferrableInsertLock(deferrableInsertLock), d_parseECS(parseECS)
 {
+  if (d_maxEntries == 0) {
+    throw std::runtime_error("Trying to create a 0-sized packet-cache");
+  }
+
   d_shards.resize(d_shardCount);
 
   /* we reserve maxEntries + 1 to avoid rehashing from occurring
@@ -41,18 +44,6 @@ DNSDistPacketCache::DNSDistPacketCache(size_t maxEntries, uint32_t maxTTL, uint3
   }
 }
 
-DNSDistPacketCache::~DNSDistPacketCache()
-{
-  try {
-    vector<std::unique_ptr<WriteLock>> locks;
-    for (auto& shard : d_shards) {
-      locks.push_back(std::make_unique<WriteLock>(&shard.d_lock));
-    }
-  }
-  catch(...) {
-  }
-}
-
 bool DNSDistPacketCache::getClientSubnet(const PacketBuffer& packet, size_t qnameWireLength, boost::optional<Netmask>& subnet)
 {
   uint16_t optRDPosition;
@@ -79,9 +70,9 @@ bool DNSDistPacketCache::getClientSubnet(const PacketBuffer& packet, size_t qnam
   return false;
 }
 
-bool DNSDistPacketCache::cachedValueMatches(const CacheValue& cachedValue, uint16_t queryFlags, const DNSName& qname, uint16_t qtype, uint16_t qclass, bool tcp, bool dnssecOK, const boost::optional<Netmask>& subnet) const
+bool DNSDistPacketCache::cachedValueMatches(const CacheValue& cachedValue, uint16_t queryFlags, const DNSName& qname, uint16_t qtype, uint16_t qclass, bool receivedOverUDP, bool dnssecOK, const boost::optional<Netmask>& subnet) const
 {
-  if (cachedValue.queryFlags != queryFlags || cachedValue.dnssecOK != dnssecOK || cachedValue.tcp != tcp || cachedValue.qtype != qtype || cachedValue.qclass != qclass || cachedValue.qname != qname) {
+  if (cachedValue.queryFlags != queryFlags || cachedValue.dnssecOK != dnssecOK || cachedValue.receivedOverUDP != receivedOverUDP || cachedValue.qtype != qtype || cachedValue.qclass != qclass || cachedValue.qname != qname) {
     return false;
   }
 
@@ -92,9 +83,8 @@ bool DNSDistPacketCache::cachedValueMatches(const CacheValue& cachedValue, uint1
   return true;
 }
 
-void DNSDistPacketCache::insertLocked(CacheShard& shard, uint32_t key, CacheValue& newValue)
+void DNSDistPacketCache::insertLocked(CacheShard& shard, std::unordered_map<uint32_t,CacheValue>& map, uint32_t key, CacheValue& newValue)
 {
-  auto& map = shard.d_map;
   /* check again now that we hold the lock to prevent a race */
   if (map.size() >= (d_maxEntries / d_shardCount)) {
     return;
@@ -114,7 +104,7 @@ void DNSDistPacketCache::insertLocked(CacheShard& shard, uint32_t key, CacheValu
   CacheValue& value = it->second;
   bool wasExpired = value.validity <= newValue.added;
 
-  if (!wasExpired && !cachedValueMatches(value, newValue.queryFlags, newValue.qname, newValue.qtype, newValue.qclass, newValue.tcp, newValue.dnssecOK, newValue.subnet)) {
+  if (!wasExpired && !cachedValueMatches(value, newValue.queryFlags, newValue.qname, newValue.qtype, newValue.qclass, newValue.receivedOverUDP, newValue.dnssecOK, newValue.subnet)) {
     d_insertCollisions++;
     return;
   }
@@ -127,7 +117,7 @@ void DNSDistPacketCache::insertLocked(CacheShard& shard, uint32_t key, CacheValu
   value = newValue;
 }
 
-void DNSDistPacketCache::insert(uint32_t key, const boost::optional<Netmask>& subnet, uint16_t queryFlags, bool dnssecOK, const DNSName& qname, uint16_t qtype, uint16_t qclass, const PacketBuffer& response, bool tcp, uint8_t rcode, boost::optional<uint32_t> tempFailureTTL)
+void DNSDistPacketCache::insert(uint32_t key, const boost::optional<Netmask>& subnet, uint16_t queryFlags, bool dnssecOK, const DNSName& qname, uint16_t qtype, uint16_t qclass, const PacketBuffer& response, bool receivedOverUDP, uint8_t rcode, boost::optional<uint32_t> tempFailureTTL)
 {
   if (response.size() < sizeof(dnsheader)) {
     return;
@@ -179,7 +169,7 @@ void DNSDistPacketCache::insert(uint32_t key, const boost::optional<Netmask>& su
   newValue.len = response.size();
   newValue.validity = newValidity;
   newValue.added = now;
-  newValue.tcp = tcp;
+  newValue.receivedOverUDP = receivedOverUDP;
   newValue.dnssecOK = dnssecOK;
   newValue.value = std::string(response.begin(), response.end());
   newValue.subnet = subnet;
@@ -187,25 +177,25 @@ void DNSDistPacketCache::insert(uint32_t key, const boost::optional<Netmask>& su
   auto& shard = d_shards.at(shardIndex);
 
   if (d_deferrableInsertLock) {
-    TryWriteLock w(&shard.d_lock);
+    auto w = shard.d_map.try_write_lock();
 
-    if (!w.gotIt()) {
+    if (!w.owns_lock()) {
       d_deferredInserts++;
       return;
     }
-    insertLocked(shard, key, newValue);
+    insertLocked(shard, *w, key, newValue);
   }
   else {
-    WriteLock w(&shard.d_lock);
+    auto w = shard.d_map.write_lock();
 
-    insertLocked(shard, key, newValue);
+    insertLocked(shard, *w, key, newValue);
   }
 }
 
-bool DNSDistPacketCache::get(DNSQuestion& dq, uint16_t queryId, uint32_t* keyOut, boost::optional<Netmask>& subnet, bool dnssecOK, uint32_t allowExpired, bool skipAging)
+bool DNSDistPacketCache::get(DNSQuestion& dq, uint16_t queryId, uint32_t* keyOut, boost::optional<Netmask>& subnet, bool dnssecOK, bool receivedOverUDP, uint32_t allowExpired, bool skipAging)
 {
   const auto& dnsQName = dq.qname->getStorage();
-  uint32_t key = getKey(dnsQName, dq.qname->wirelength(), dq.getData(), dq.tcp);
+  uint32_t key = getKey(dnsQName, dq.qname->wirelength(), dq.getData(), receivedOverUDP);
 
   if (keyOut) {
     *keyOut = key;
@@ -221,16 +211,15 @@ bool DNSDistPacketCache::get(DNSQuestion& dq, uint16_t queryId, uint32_t* keyOut
   bool stale = false;
   auto& response = dq.getMutableData();
   auto& shard = d_shards.at(shardIndex);
-  auto& map = shard.d_map;
   {
-    TryReadLock r(&shard.d_lock);
-    if (!r.gotIt()) {
+    auto map = shard.d_map.try_read_lock();
+    if (!map.owns_lock()) {
       d_deferredLookups++;
       return false;
     }
 
-    std::unordered_map<uint32_t,CacheValue>::const_iterator it = map.find(key);
-    if (it == map.end()) {
+    std::unordered_map<uint32_t,CacheValue>::const_iterator it = map->find(key);
+    if (it == map->end()) {
       d_misses++;
       return false;
     }
@@ -251,7 +240,7 @@ bool DNSDistPacketCache::get(DNSQuestion& dq, uint16_t queryId, uint32_t* keyOut
     }
 
     /* check for collision */
-    if (!cachedValueMatches(value, *(getFlagsFromDNSHeader(dq.getHeader())), *dq.qname, dq.qtype, dq.qclass, dq.tcp, dnssecOK, subnet)) {
+    if (!cachedValueMatches(value, *(getFlagsFromDNSHeader(dq.getHeader())), *dq.qname, dq.qtype, dq.qclass, receivedOverUDP, dnssecOK, subnet)) {
       d_lookupCollisions++;
       return false;
     }
@@ -310,19 +299,18 @@ size_t DNSDistPacketCache::purgeExpired(size_t upTo, const time_t now)
   size_t removed = 0;
 
   for (auto& shard : d_shards) {
-    WriteLock w(&shard.d_lock);
-    auto& map = shard.d_map;
-    if (map.size() <= maxPerShard) {
+    auto map = shard.d_map.write_lock();
+    if (map->size() <= maxPerShard) {
       continue;
     }
 
-    size_t toRemove = map.size() - maxPerShard;
+    size_t toRemove = map->size() - maxPerShard;
 
-    for (auto it = map.begin(); toRemove > 0 && it != map.end(); ) {
+    for (auto it = map->begin(); toRemove > 0 && it != map->end(); ) {
       const CacheValue& value = it->second;
 
       if (value.validity <= now) {
-        it = map.erase(it);
+        it = map->erase(it);
         --toRemove;
         --shard.d_entriesCount;
         ++removed;
@@ -347,27 +335,26 @@ size_t DNSDistPacketCache::expunge(size_t upTo)
   size_t removed = 0;
 
   for (auto& shard : d_shards) {
-    WriteLock w(&shard.d_lock);
-    auto& map = shard.d_map;
+    auto map = shard.d_map.write_lock();
 
-    if (map.size() <= maxPerShard) {
+    if (map->size() <= maxPerShard) {
       continue;
     }
 
-    size_t toRemove = map.size() - maxPerShard;
+    size_t toRemove = map->size() - maxPerShard;
 
-    auto beginIt = map.begin();
+    auto beginIt = map->begin();
     auto endIt = beginIt;
 
-    if (map.size() >= toRemove) {
+    if (map->size() >= toRemove) {
       std::advance(endIt, toRemove);
-      map.erase(beginIt, endIt);
+      map->erase(beginIt, endIt);
       shard.d_entriesCount -= toRemove;
       removed += toRemove;
     }
     else {
-      removed += map.size();
-      map.clear();
+      removed += map->size();
+      map->clear();
       shard.d_entriesCount = 0;
     }
   }
@@ -380,14 +367,13 @@ size_t DNSDistPacketCache::expungeByName(const DNSName& name, uint16_t qtype, bo
   size_t removed = 0;
 
   for (auto& shard : d_shards) {
-    WriteLock w(&shard.d_lock);
-    auto& map = shard.d_map;
+    auto map = shard.d_map.write_lock();
 
-    for(auto it = map.begin(); it != map.end(); ) {
+    for(auto it = map->begin(); it != map->end(); ) {
       const CacheValue& value = it->second;
 
       if ((value.qname == name || (suffixMatch && value.qname.isPartOf(name))) && (qtype == QType::ANY || qtype == value.qtype)) {
-        it = map.erase(it);
+        it = map->erase(it);
         --shard.d_entriesCount;
         ++removed;
       } else {
@@ -420,7 +406,7 @@ uint32_t DNSDistPacketCache::getMinTTL(const char* packet, uint16_t length, bool
   return getDNSPacketMinTTL(packet, length, seenNoDataSOA);
 }
 
-uint32_t DNSDistPacketCache::getKey(const DNSName::string_t& qname, size_t qnameWireLength, const PacketBuffer& packet, bool tcp)
+uint32_t DNSDistPacketCache::getKey(const DNSName::string_t& qname, size_t qnameWireLength, const PacketBuffer& packet, bool receivedOverUDP)
 {
   uint32_t result = 0;
   /* skip the query ID */
@@ -434,15 +420,15 @@ uint32_t DNSDistPacketCache::getKey(const DNSName::string_t& qname, size_t qname
     throw std::range_error("Computing packet cache key for an invalid packet (" + std::to_string(packet.size()) + " < " + std::to_string(sizeof(dnsheader) + qnameWireLength) + ")");
   }
   if (packet.size() > ((sizeof(dnsheader) + qnameWireLength))) {
-    if (!d_cookieHashing) {
-      /* skip EDNS Cookie options if any */
-      result = PacketCache::hashAfterQname(pdns_string_view(reinterpret_cast<const char*>(packet.data()), packet.size()), result, sizeof(dnsheader) + qnameWireLength, false);
+    if (!d_optionsToSkip.empty()) {
+      /* skip EDNS options if any */
+      result = PacketCache::hashAfterQname(pdns_string_view(reinterpret_cast<const char*>(packet.data()), packet.size()), result, sizeof(dnsheader) + qnameWireLength, d_optionsToSkip);
     }
     else {
       result = burtle(&packet.at(sizeof(dnsheader) + qnameWireLength), packet.size() - (sizeof(dnsheader) + qnameWireLength), result);
     }
   }
-  result = burtle((const unsigned char*) &tcp, sizeof(tcp), result);
+  result = burtle((const unsigned char*) &receivedOverUDP, sizeof(receivedOverUDP), result);
   return result;
 }
 
@@ -473,10 +459,9 @@ uint64_t DNSDistPacketCache::dump(int fd)
   uint64_t count = 0;
   time_t now = time(nullptr);
   for (auto& shard : d_shards) {
-    ReadLock w(&shard.d_lock);
-    auto& map = shard.d_map;
+    auto map = shard.d_map.read_lock();
 
-    for (const auto& entry : map) {
+    for (const auto& entry : *map) {
       const CacheValue& value = entry.second;
       count++;
 
@@ -488,7 +473,7 @@ uint64_t DNSDistPacketCache::dump(int fd)
           rcode = dh.rcode;
         }
 
-        fprintf(fp.get(), "%s %" PRId64 " %s ; rcode %" PRIu8 ", key %" PRIu32 ", length %" PRIu16 ", tcp %d, added %" PRId64 "\n", value.qname.toString().c_str(), static_cast<int64_t>(value.validity - now), QType(value.qtype).getName().c_str(), rcode, entry.first, value.len, value.tcp, static_cast<int64_t>(value.added));
+        fprintf(fp.get(), "%s %" PRId64 " %s ; rcode %" PRIu8 ", key %" PRIu32 ", length %" PRIu16 ", received over UDP %d, added %" PRId64 "\n", value.qname.toString().c_str(), static_cast<int64_t>(value.validity - now), QType(value.qtype).toString().c_str(), rcode, entry.first, value.len, value.receivedOverUDP, static_cast<int64_t>(value.added));
       }
       catch(...) {
         fprintf(fp.get(), "; error printing '%s'\n", value.qname.empty() ? "EMPTY" : value.qname.toString().c_str());
@@ -498,3 +483,8 @@ uint64_t DNSDistPacketCache::dump(int fd)
 
   return count;
 }
+
+void DNSDistPacketCache::setSkippedOptions(const std::unordered_set<uint16_t>& optionsToSkip)
+{
+  d_optionsToSkip = optionsToSkip;
+}
