@@ -107,6 +107,7 @@ void carbonDumpThread()
             str<<base<<"tcpmaxconcurrentconnections" << ' '<< state->tcpMaxConcurrentConnections.load() << " " << now << "\r\n";
             str<<base<<"tcpnewconnections" << ' '<< state->tcpNewConnections.load() << " " << now << "\r\n";
             str<<base<<"tcpreusedconnections" << ' '<< state->tcpReusedConnections.load() << " " << now << "\r\n";
+            str<<base<<"tlsresumptions" << ' '<< state->tlsResumptions.load() << " " << now << "\r\n";
             str<<base<<"tcpavgqueriesperconnection" << ' '<< state->tcpAvgQueriesPerConnection.load() << " " << now << "\r\n";
             str<<base<<"tcpavgconnectionduration" << ' '<< state->tcpAvgConnectionDuration.load() << " " << now << "\r\n";
           }
@@ -238,14 +239,14 @@ void carbonDumpThread()
 #endif /* HAVE_DNS_OVER_HTTPS */
 
           {
-            WriteLock wl(&g_qcount.queryLock);
             std::string qname;
-            for(auto &record: g_qcount.records) {
+            auto records = g_qcount.records.write_lock();
+            for (const auto &record : *records) {
               qname = record.first;
               boost::replace_all(qname, ".", "_");
               str<<"dnsdist.querycount." << qname << ".queries " << record.second << " " << now << "\r\n";
             }
-            g_qcount.records.clear();
+            records->clear();
           }
 
           const string msg = str.str();
