@@ -1,9 +1,9 @@
 
 #include "dnsdist.hh"
 
-DNSResponse makeDNSResponseFromIDState(IDState& ids, PacketBuffer& data, bool isTCP)
+DNSResponse makeDNSResponseFromIDState(IDState& ids, PacketBuffer& data)
 {
-  DNSResponse dr(&ids.qname, ids.qtype, ids.qclass, &ids.origDest, &ids.origRemote, data, isTCP, &ids.sentTime.d_start);
+  DNSResponse dr(&ids.qname, ids.qtype, ids.qclass, &ids.origDest, &ids.origRemote, data, ids.protocol, &ids.sentTime.d_start);
   dr.origFlags = ids.origFlags;
   dr.cacheFlags = ids.cacheFlags;
   dr.ecsAdded = ids.ecsAdded;
@@ -14,6 +14,7 @@ DNSResponse makeDNSResponseFromIDState(IDState& ids, PacketBuffer& data, bool is
   dr.skipCache = ids.skipCache;
   dr.cacheKey = ids.cacheKey;
   dr.cacheKeyNoECS = ids.cacheKeyNoECS;
+  dr.cacheKeyUDP = ids.cacheKeyUDP;
   dr.dnssecOK = ids.dnssecOK;
   dr.tempFailureTTL = ids.tempFailureTTL;
   dr.qTag = std::move(ids.qTag);
@@ -38,19 +39,21 @@ void setIDStateFromDNSQuestion(IDState& ids, DNSQuestion& dq, DNSName&& qname)
   ids.qname = std::move(qname);
   ids.qtype = dq.qtype;
   ids.qclass = dq.qclass;
+  ids.protocol = dq.protocol;
   ids.delayMsec = dq.delayMsec;
   ids.tempFailureTTL = dq.tempFailureTTL;
   ids.origFlags = dq.origFlags;
   ids.cacheFlags = dq.cacheFlags;
   ids.cacheKey = dq.cacheKey;
   ids.cacheKeyNoECS = dq.cacheKeyNoECS;
+  ids.cacheKeyUDP = dq.cacheKeyUDP;
   ids.subnet = dq.subnet;
   ids.skipCache = dq.skipCache;
   ids.packetCache = dq.packetCache;
   ids.ednsAdded = dq.ednsAdded;
   ids.ecsAdded = dq.ecsAdded;
   ids.useZeroScope = dq.useZeroScope;
-  ids.qTag = dq.qTag;
+  ids.qTag = std::move(dq.qTag);
   ids.dnssecOK = dq.dnssecOK;
   ids.uniqueId = std::move(dq.uniqueId);
 
