@@ -99,3 +99,8 @@ uint64_t dnsdist_ffi_stat_node_get_children_bytes_count(const dnsdist_ffi_stat_n
 {
   return node->children.bytes;
 }
+
+void dnsdist_ffi_state_node_set_reason(dnsdist_ffi_stat_node_t* node, const char* reason, size_t reasonSize)
+{
+  node->reason = std::string(reason, reasonSize);
+}
