@@ -22,7 +22,9 @@
 #include "dnsdist.hh"
 #include "dnsdist-lua.hh"
 #include "dnsdist-dynblocks.hh"
+#include "dnsdist-nghttp2.hh"
 #include "dnsdist-rings.hh"
+#include "dnsdist-tcp.hh"
 
 #include "statnode.hh"
 
@@ -33,9 +35,9 @@ static std::unordered_map<unsigned int, vector<boost::variant<string,double>>> g
   unsigned int total=0;
   {
     for (const auto& shard : g_rings.d_shards) {
-      std::lock_guard<std::mutex> rl(shard->respLock);
-      if(!labels) {
-        for(const auto& a : shard->respRing) {
+      auto rl = shard->respRing.lock();
+      if (!labels) {
+        for(const auto& a : *rl) {
           if(!pred(a))
             continue;
           counts[a.name]++;
@@ -44,7 +46,7 @@ static std::unordered_map<unsigned int, vector<boost::variant<string,double>>> g
       }
       else {
         unsigned int lab = *labels;
-        for(const auto& a : shard->respRing) {
+        for(const auto& a : *rl) {
           if(!pred(a))
             continue;
 
@@ -60,7 +62,7 @@ static std::unordered_map<unsigned int, vector<boost::variant<string,double>>> g
   vector<pair<unsigned int, DNSName>> rcounts;
   rcounts.reserve(counts.size());
   for(const auto& c : counts)
-    rcounts.push_back(make_pair(c.second, c.first.makeLowerCase()));
+    rcounts.emplace_back(c.second, c.first.makeLowerCase());
 
   sort(rcounts.begin(), rcounts.end(), [](const decltype(rcounts)::value_type& a,
                                           const decltype(rcounts)::value_type& b) {
@@ -115,9 +117,9 @@ static void statNodeRespRing(statvisitor_t visitor, unsigned int seconds)
 
   StatNode root;
   for (const auto& shard : g_rings.d_shards) {
-    std::lock_guard<std::mutex> rl(shard->respLock);
+    auto rl = shard->respRing.lock();
 
-    for(const auto& c : shard->respRing) {
+    for(const auto& c : *rl) {
       if (now < c.when)
         continue;
 
@@ -139,16 +141,16 @@ static vector<pair<unsigned int, std::unordered_map<string,string> > > getRespRi
   vector<pair<unsigned int, entry_t > > ret;
 
   for (const auto& shard : g_rings.d_shards) {
-    std::lock_guard<std::mutex> rl(shard->respLock);
+    auto rl = shard->respRing.lock();
 
     entry_t e;
     unsigned int count=1;
-    for(const auto& c : shard->respRing) {
+    for(const auto& c : *rl) {
       if(rcode && (rcode.get() != c.dh.rcode))
         continue;
       e["qname"]=c.name.toString();
       e["rcode"]=std::to_string(c.dh.rcode);
-      ret.push_back(std::make_pair(count,e));
+      ret.emplace_back(count, e);
       count++;
     }
   }
@@ -167,8 +169,8 @@ static counts_t exceedRespGen(unsigned int rate, int seconds, std::function<void
   counts.reserve(g_rings.getNumberOfResponseEntries());
 
   for (const auto& shard : g_rings.d_shards) {
-    std::lock_guard<std::mutex> rl(shard->respLock);
-    for(const auto& c : shard->respRing) {
+    auto rl = shard->respRing.lock();
+    for(const auto& c : *rl) {
 
       if(seconds && c.when < cutoff)
         continue;
@@ -196,8 +198,8 @@ static counts_t exceedQueryGen(unsigned int rate, int seconds, std::function<voi
   counts.reserve(g_rings.getNumberOfQueryEntries());
 
   for (const auto& shard : g_rings.d_shards) {
-    std::lock_guard<std::mutex> rl(shard->queryLock);
-    for(const auto& c : shard->queryRing) {
+    auto rl = shard->queryRing.lock();
+    for(const auto& c : *rl) {
       if(seconds && c.when < cutoff)
         continue;
       if(now < c.when)
@@ -239,8 +241,8 @@ void setupLuaInspection(LuaContext& luaCtx)
       unsigned int total=0;
       {
         for (const auto& shard : g_rings.d_shards) {
-          std::lock_guard<std::mutex> rl(shard->queryLock);
-          for(const auto& c : shard->queryRing) {
+          auto rl = shard->queryRing.lock();
+          for(const auto& c : *rl) {
             counts[c.requestor]++;
             total++;
           }
@@ -249,7 +251,7 @@ void setupLuaInspection(LuaContext& luaCtx)
       vector<pair<unsigned int, ComboAddress>> rcounts;
       rcounts.reserve(counts.size());
       for(const auto& c : counts)
-	rcounts.push_back(make_pair(c.second, c.first));
+        rcounts.emplace_back(c.second, c.first);
 
       sort(rcounts.begin(), rcounts.end(), [](const decltype(rcounts)::value_type& a,
 					      const decltype(rcounts)::value_type& b) {
@@ -272,8 +274,8 @@ void setupLuaInspection(LuaContext& luaCtx)
       unsigned int total=0;
       if(!labels) {
         for (const auto& shard : g_rings.d_shards) {
-          std::lock_guard<std::mutex> rl(shard->queryLock);
-          for(const auto& a : shard->queryRing) {
+          auto rl = shard->queryRing.lock();
+          for(const auto& a : *rl) {
             counts[a.name]++;
             total++;
           }
@@ -282,8 +284,8 @@ void setupLuaInspection(LuaContext& luaCtx)
       else {
 	unsigned int lab = *labels;
         for (const auto& shard : g_rings.d_shards) {
-          std::lock_guard<std::mutex> rl(shard->queryLock);
-          for(auto a : shard->queryRing) {
+          auto rl = shard->queryRing.lock();
+          for(auto a : *rl) {
             a.name.trimToLabels(lab);
             counts[a.name]++;
             total++;
@@ -294,7 +296,7 @@ void setupLuaInspection(LuaContext& luaCtx)
       vector<pair<unsigned int, DNSName>> rcounts;
       rcounts.reserve(counts.size());
       for(const auto& c : counts)
-	rcounts.push_back(make_pair(c.second, c.first.makeLowerCase()));
+        rcounts.emplace_back(c.second, c.first.makeLowerCase());
 
       sort(rcounts.begin(), rcounts.end(), [](const decltype(rcounts)::value_type& a,
 					      const decltype(rcounts)::value_type& b) {
@@ -330,8 +332,8 @@ void setupLuaInspection(LuaContext& luaCtx)
       rings.reserve(g_rings.getNumberOfShards());
       for (const auto& shard : g_rings.d_shards) {
         {
-          std::lock_guard<std::mutex> rl(shard->respLock);
-          rings.push_back(shard->respRing);
+          auto rl = shard->respRing.lock();
+          rings.push_back(*rl);
         }
         totalEntries += rings.back().size();
       }
@@ -426,14 +428,14 @@ void setupLuaInspection(LuaContext& luaCtx)
       rr.reserve(g_rings.getNumberOfResponseEntries());
       for (const auto& shard : g_rings.d_shards) {
         {
-          std::lock_guard<std::mutex> rl(shard->queryLock);
-          for (const auto& entry : shard->queryRing) {
+          auto rl = shard->queryRing.lock();
+          for (const auto& entry : *rl) {
             qr.push_back(entry);
           }
         }
         {
-          std::lock_guard<std::mutex> rl(shard->respLock);
-          for (const auto& entry : shard->respRing) {
+          auto rl = shard->respRing.lock();
+          for (const auto& entry : *rl) {
             rr.push_back(entry);
           }
         }
@@ -453,8 +455,8 @@ void setupLuaInspection(LuaContext& luaCtx)
 
       std::multimap<struct timespec, string> out;
 
-      boost::format      fmt("%-7.1f %-47s %-12s %-5d %-25s %-5s %-6.1f %-2s %-2s %-2s %-s\n");
-      g_outputBuffer+= (fmt % "Time" % "Client" % "Server" % "ID" % "Name" % "Type" % "Lat." % "TC" % "RD" % "AA" % "Rcode").str();
+      boost::format      fmt("%-7.1f %-47s %-12s %-12s %-5d %-25s %-5s %-6.1f %-2s %-2s %-2s %-s\n");
+      g_outputBuffer+= (fmt % "Time" % "Client" % "Protocol" % "Server" % "ID" % "Name" % "Type" % "Lat." % "TC" % "RD" % "AA" % "Rcode").str();
 
       if(msec==-1) {
         for(const auto& c : qr) {
@@ -476,7 +478,7 @@ void setupLuaInspection(LuaContext& luaCtx)
             if (c.dh.opcode != 0) {
               extra = " (" + Opcode::to_s(c.dh.opcode) + ")";
             }
-            out.insert(make_pair(c.when, (fmt % DiffTime(now, c.when) % c.requestor.toStringWithPort() % "" % htons(c.dh.id) % c.name.toString() % qt.getName()  % "" % (c.dh.tc ? "TC" : "") % (c.dh.rd? "RD" : "") % (c.dh.aa? "AA" : "") % ("Question" + extra)).str() )) ;
+            out.emplace(c.when, (fmt % DiffTime(now, c.when) % c.requestor.toStringWithPort() % dnsdist::Protocol(c.protocol).toString() % "" % htons(c.dh.id) % c.name.toString() % qt.toString() % "" % (c.dh.tc ? "TC" : "") % (c.dh.rd ? "RD" : "") % (c.dh.aa ? "AA" : "") % ("Question" + extra)).str());
 
             if(limit && *limit==++num)
               break;
@@ -514,10 +516,10 @@ void setupLuaInspection(LuaContext& luaCtx)
           }
 
           if (c.usec != std::numeric_limits<decltype(c.usec)>::max()) {
-            out.insert(make_pair(c.when, (fmt % DiffTime(now, c.when) % c.requestor.toStringWithPort() % c.ds.toStringWithPort() % htons(c.dh.id) % c.name.toString()  % qt.getName()  % (c.usec/1000.0) % (c.dh.tc ? "TC" : "") % (c.dh.rd? "RD" : "") % (c.dh.aa? "AA" : "") % (RCode::to_s(c.dh.rcode) + extra)).str()  )) ;
+            out.emplace(c.when, (fmt % DiffTime(now, c.when) % c.requestor.toStringWithPort() % dnsdist::Protocol(c.protocol).toString() % c.ds.toStringWithPort() % htons(c.dh.id) % c.name.toString() % qt.toString() % (c.usec / 1000.0) % (c.dh.tc ? "TC" : "") % (c.dh.rd ? "RD" : "") % (c.dh.aa ? "AA" : "") % (RCode::to_s(c.dh.rcode) + extra)).str());
           }
           else {
-            out.insert(make_pair(c.when, (fmt % DiffTime(now, c.when) % c.requestor.toStringWithPort() % c.ds.toStringWithPort() % htons(c.dh.id) % c.name.toString()  % qt.getName()  % "T.O" % (c.dh.tc ? "TC" : "") % (c.dh.rd? "RD" : "") % (c.dh.aa? "AA" : "") % (RCode::to_s(c.dh.rcode) + extra)).str()  )) ;
+            out.emplace(c.when, (fmt % DiffTime(now, c.when) % c.requestor.toStringWithPort() % dnsdist::Protocol(c.protocol).toString() % c.ds.toStringWithPort() % htons(c.dh.id) % c.name.toString() % qt.toString() % "T.O" % (c.dh.tc ? "TC" : "") % (c.dh.rd ? "RD" : "") % (c.dh.aa ? "AA" : "") % (RCode::to_s(c.dh.rcode) + extra)).str());
           }
 
           if (limit && *limit == ++num) {
@@ -544,8 +546,8 @@ void setupLuaInspection(LuaContext& luaCtx)
       unsigned int size=0;
       {
         for (const auto& shard : g_rings.d_shards) {
-          std::lock_guard<std::mutex> rl(shard->respLock);
-          for(const auto& r : shard->respRing) {
+          auto rl = shard->respRing.lock();
+          for(const auto& r : *rl) {
             /* skip actively discovered timeouts */
             if (r.usec == std::numeric_limits<unsigned int>::max())
               continue;
@@ -597,9 +599,6 @@ void setupLuaInspection(LuaContext& luaCtx)
       ret << (fmt % g_tcpclientthreads->getThreadsCount() % (g_maxTCPClientThreads ? *g_maxTCPClientThreads : 0) % g_tcpclientthreads->getQueuedCount() % g_maxTCPQueuedConnections) << endl;
       ret << endl;
 
-      ret << "Query distribution mode is: " << std::string(g_useTCPSinglePipe ? "single queue" : "per-thread queues") << endl;
-      ret << endl;
-
       ret << "Frontends:" << endl;
       fmt = boost::format("%-3d %-20.20s %-20d %-20d %-20d %-25d %-20d %-20d %-20d %-20f %-20f %-20d %-20d %-25d %-25d %-15d %-15d %-15d %-15d %-15d");
       ret << (fmt % "#" % "Address" % "Connections" % "Max concurrent conn" % "Died reading query" % "Died sending response" % "Gave up" % "Client timeouts" % "Downstream timeouts" % "Avg queries/conn" % "Avg duration" % "TLS new sessions" % "TLS Resumptions" % "TLS unknown ticket keys" % "TLS inactive ticket keys" % "TLS 1.0" % "TLS 1.1" % "TLS 1.2" % "TLS 1.3" % "TLS other") << endl;
@@ -612,13 +611,13 @@ void setupLuaInspection(LuaContext& luaCtx)
       ret << endl;
 
       ret << "Backends:" << endl;
-      fmt = boost::format("%-3d %-20.20s %-20.20s %-20d %-20d %-25d %-20d %-20d %-20d %-20d %-20d %-20d %-20d %-20f %-20f");
-      ret << (fmt % "#" % "Name" % "Address" % "Connections" % " Max concurrent conn" % "Died sending query" % "Died reading response" % "Gave up" % "Read timeouts" % "Write timeouts" % "Connect timeouts" % "Total connections" % "Reused connections" % "Avg queries/conn" % "Avg duration") << endl;
+      fmt = boost::format("%-3d %-20.20s %-20.20s %-20d %-20d %-25d %-25d %-20d %-20d %-20d %-20d %-20d %-20d %-20d %-20f %-20f");
+      ret << (fmt % "#" % "Name" % "Address" % "Connections" % "Max concurrent conn" % "Died sending query" % "Died reading response" % "Gave up" % "Read timeouts" % "Write timeouts" % "Connect timeouts" % "Total connections" % "Reused connections" % "TLS resumptions" % "Avg queries/conn" % "Avg duration") << endl;
 
       auto states = g_dstates.getLocal();
       counter = 0;
       for(const auto& s : *states) {
-        ret << (fmt % counter % s->getName() % s->remote.toStringWithPort() % s->tcpCurrentConnections % s->tcpMaxConcurrentConnections % s->tcpDiedSendingQuery % s->tcpDiedReadingResponse % s->tcpGaveUp % s->tcpReadTimeouts % s->tcpWriteTimeouts % s->tcpConnectTimeouts % s->tcpNewConnections % s->tcpReusedConnections % s->tcpAvgQueriesPerConnection % s->tcpAvgConnectionDuration) << endl;
+        ret << (fmt % counter % s->getName() % s->remote.toStringWithPort() % s->tcpCurrentConnections % s->tcpMaxConcurrentConnections % s->tcpDiedSendingQuery % s->tcpDiedReadingResponse % s->tcpGaveUp % s->tcpReadTimeouts % s->tcpWriteTimeouts % s->tcpConnectTimeouts % s->tcpNewConnections % s->tcpReusedConnections % s->tlsResumptions % s->tcpAvgQueriesPerConnection % s->tcpAvgConnectionDuration) << endl;
         ++counter;
       }
 
@@ -662,11 +661,16 @@ void setupLuaInspection(LuaContext& luaCtx)
     g_tcpStatesDumpRequested += g_tcpclientthreads->getThreadsCount();
   });
 
+  luaCtx.writeFunction("requestDoHStatesDump", [] {
+    setLuaNoSideEffect();
+    g_dohStatesDumpRequested += g_dohClientThreads->getThreadsCount();
+  });
+
   luaCtx.writeFunction("dumpStats", [] {
       setLuaNoSideEffect();
       vector<string> leftcolumn, rightcolumn;
 
-      boost::format fmt("%-23s\t%+11s");
+      boost::format fmt("%-35s\t%+11s");
       g_outputBuffer.clear();
       auto entries = g_stats.entries;
       sort(entries.begin(), entries.end(),
@@ -690,7 +694,7 @@ void setupLuaInspection(LuaContext& luaCtx)
       }
 
       auto leftiter=leftcolumn.begin(), rightiter=rightcolumn.begin();
-      boost::format clmn("%|0t|%1% %|39t|%2%\n");
+      boost::format clmn("%|0t|%1% %|51t|%2%\n");
 
       for(;leftiter != leftcolumn.end() || rightiter != rightcolumn.end();) {
 	string lentry, rentry;
@@ -792,6 +796,23 @@ void setupLuaInspection(LuaContext& luaCtx)
         group->setQTypeRate(qtype, rate, warningRate ? *warningRate : 0, seconds, reason, blockDuration, action ? *action : DNSAction::Action::None);
       }
     });
+  luaCtx.registerFunction<void(std::shared_ptr<DynBlockRulesGroup>::*)(uint8_t, uint8_t, uint8_t)>("setMasks", [](std::shared_ptr<DynBlockRulesGroup>& group, uint8_t v4, uint8_t v6, uint8_t port) {
+      if (group) {
+        if (v4 > 32) {
+          throw std::runtime_error("Trying to set an invalid IPv4 mask (" + std::to_string(v4) + ") to a Dynamic Block object");
+        }
+        if (v6 > 128) {
+          throw std::runtime_error("Trying to set an invalid IPv6 mask (" + std::to_string(v6) + ") to a Dynamic Block object");
+        }
+        if (port > 16) {
+          throw std::runtime_error("Trying to set an invalid port mask (" + std::to_string(port) + ") to a Dynamic Block object");
+        }
+        if (port > 0 && v4 != 32) {
+          throw std::runtime_error("Setting a non-zero port mask for Dynamic Blocks while only considering parts of IPv4 addresses does not make sense");
+        }
+        group->setMasks(v4, v6, port);
+      }
+    });
   luaCtx.registerFunction<void(std::shared_ptr<DynBlockRulesGroup>::*)(boost::variant<std::string, std::vector<std::pair<int, std::string>>, NetmaskGroup>)>("excludeRange", [](std::shared_ptr<DynBlockRulesGroup>& group, boost::variant<std::string, std::vector<std::pair<int, std::string>>, NetmaskGroup> ranges) {
       if (ranges.type() == typeid(std::vector<std::pair<int, std::string>>)) {
         for (const auto& range : *boost::get<std::vector<std::pair<int, std::string>>>(&ranges)) {
