@@ -28,8 +28,8 @@ size_t Rings::numDistinctRequestors()
 {
   std::set<ComboAddress, ComboAddress::addressOnlyLessThan> s;
   for (const auto& shard : d_shards) {
-    std::lock_guard<std::mutex> rl(shard->queryLock);
-    for(const auto& q : shard->queryRing) {
+    auto rl = shard->queryRing.lock();
+    for (const auto& q : *rl) {
       s.insert(q.requestor);
     }
   }
@@ -42,16 +42,16 @@ std::unordered_map<int, vector<boost::variant<string,double>>> Rings::getTopBand
   uint64_t total=0;
   for (const auto& shard : d_shards) {
     {
-      std::lock_guard<std::mutex> rl(shard->queryLock);
-      for(const auto& q : shard->queryRing) {
-        counts[q.requestor]+=q.size;
+      auto rl = shard->queryRing.lock();
+      for(const auto& q : *rl) {
+        counts[q.requestor] += q.size;
         total+=q.size;
       }
     }
     {
-      std::lock_guard<std::mutex> rl(shard->respLock);
-      for(const auto& r : shard->respRing) {
-        counts[r.requestor]+=r.size;
+      auto rl = shard->respRing.lock();
+      for(const auto& r : *rl) {
+        counts[r.requestor] += r.size;
         total+=r.size;
       }
     }
@@ -108,9 +108,9 @@ size_t Rings::loadFromFile(const std::string& filepath, const struct timespec& n
     vector<string> parts;
     stringtok(parts, line, " \t,");
 
-    if (parts.size() == 7) {
+    if (parts.size() == 8) {
     }
-    else if (parts.size() >= 10 && parts.size() <= 12) {
+    else if (parts.size() >= 11 && parts.size() <= 13) {
       isResponse = true;
     }
     else {
@@ -138,7 +138,7 @@ size_t Rings::loadFromFile(const std::string& filepath, const struct timespec& n
 
     ComboAddress from(parts.at(idx++));
     ComboAddress to;
-
+    dnsdist::Protocol protocol(parts.at(idx++));
     if (isResponse) {
       to = ComboAddress(parts.at(idx++));
     }
@@ -148,10 +148,10 @@ size_t Rings::loadFromFile(const std::string& filepath, const struct timespec& n
     QType qtype(QType::chartocode(parts.at(idx++).c_str()));
 
     if (isResponse) {
-      insertResponse(when, from, qname, qtype.getCode(), 0, 0, dh, to);
+      insertResponse(when, from, qname, qtype.getCode(), 0, 0, dh, to, protocol);
     }
     else {
-      insertQuery(when, from, qname, qtype.getCode(), 0, dh);
+      insertQuery(when, from, qname, qtype.getCode(), 0, dh, protocol);
     }
     ++inserted;
   }
