@@ -311,14 +311,14 @@ static int backendStatTable_handler(netsnmp_mib_handler* handler,
         break;
       case COLUMN_BACKENDOUTSTANDING:
         DNSDistSNMPAgent::setCounter64Value(request,
-                                            server->outstanding);
+                                            server->outstanding.load());
         break;
       case COLUMN_BACKENDQPSLIMIT:
         DNSDistSNMPAgent::setCounter64Value(request,
                                             server->qps.getRate());
         break;
       case COLUMN_BACKENDREUSED:
-        DNSDistSNMPAgent::setCounter64Value(request, server->reuseds);
+        DNSDistSNMPAgent::setCounter64Value(request, server->reuseds.load());
         break;
       case COLUMN_BACKENDSTATE:
       {
@@ -353,10 +353,10 @@ static int backendStatTable_handler(netsnmp_mib_handler* handler,
         break;
       }
       case COLUMN_BACKENDQPS:
-        DNSDistSNMPAgent::setCounter64Value(request, server->queryLoad);
+        DNSDistSNMPAgent::setCounter64Value(request, server->queryLoad.load());
         break;
       case COLUMN_BACKENDQUERIES:
-        DNSDistSNMPAgent::setCounter64Value(request, server->queries);
+        DNSDistSNMPAgent::setCounter64Value(request, server->queries.load());
         break;
       case COLUMN_BACKENDORDER:
         DNSDistSNMPAgent::setCounter64Value(request, server->order);
@@ -448,7 +448,7 @@ bool DNSDistSNMPAgent::sendDNSTrap(const DNSQuestion& dq, const std::string& rea
   std::string remote = dq.remote->toString();
   std::string qname = dq.qname->toStringNoDot();
   const uint32_t socketFamily = dq.remote->isIPv4() ? 1 : 2;
-  const uint32_t socketProtocol = dq.tcp ? 2 : 1;
+  const uint32_t socketProtocol = dq.overTCP() ? 2 : 1;
   const uint32_t queryType = dq.getHeader()->qr ? 2 : 1;
   const uint32_t querySize = (uint32_t) dq.getData().size();
   const uint32_t queryID = (uint32_t) ntohs(dq.getHeader()->id);
@@ -468,14 +468,14 @@ bool DNSDistSNMPAgent::sendDNSTrap(const DNSQuestion& dq, const std::string& rea
                             socketFamilyOID,
                             OID_LENGTH(socketFamilyOID),
                             ASN_INTEGER,
-                            (u_char *) &socketFamily,
+                            reinterpret_cast<const u_char*>(&socketFamily),
                             sizeof(socketFamily));
 
   snmp_varlist_add_variable(&varList,
                             socketProtocolOID,
                             OID_LENGTH(socketProtocolOID),
                             ASN_INTEGER,
-                            (u_char *) &socketProtocol,
+                            reinterpret_cast<const u_char*>(&socketProtocol),
                             sizeof(socketProtocol));
 
   snmp_varlist_add_variable(&varList,
@@ -496,21 +496,21 @@ bool DNSDistSNMPAgent::sendDNSTrap(const DNSQuestion& dq, const std::string& rea
                             queryTypeOID,
                             OID_LENGTH(queryTypeOID),
                             ASN_INTEGER,
-                            (u_char *) &queryType,
+                            reinterpret_cast<const u_char*>(&queryType),
                             sizeof(queryType));
 
   snmp_varlist_add_variable(&varList,
                             querySizeOID,
                             OID_LENGTH(querySizeOID),
                             ASN_UNSIGNED,
-                            (u_char *) &querySize,
+                            reinterpret_cast<const u_char*>(&querySize),
                             sizeof(querySize));
 
   snmp_varlist_add_variable(&varList,
                             queryIDOID,
                             OID_LENGTH(queryIDOID),
                             ASN_UNSIGNED,
-                            (u_char *) &queryID,
+                            reinterpret_cast<const u_char*>(&queryID),
                             sizeof(queryID));
 
   snmp_varlist_add_variable(&varList,
@@ -524,14 +524,14 @@ bool DNSDistSNMPAgent::sendDNSTrap(const DNSQuestion& dq, const std::string& rea
                             qClassOID,
                             OID_LENGTH(qClassOID),
                             ASN_UNSIGNED,
-                            (u_char *) &qClass,
+                            reinterpret_cast<const u_char*>(&qClass),
                             sizeof(qClass));
 
   snmp_varlist_add_variable(&varList,
                             qTypeOID,
                             OID_LENGTH(qTypeOID),
                             ASN_UNSIGNED,
-                            (u_char *) &qType,
+                            reinterpret_cast<const u_char*>(&qType),
                             sizeof(qType));
 
   snmp_varlist_add_variable(&varList,
