@@ -41,7 +41,7 @@ std::ostream & operator<<(std::ostream &os, const DNSName& d)
   return os <<d.toLogString();
 }
 
-DNSName::DNSName(const char* p)
+DNSName::DNSName(const char* p, size_t length)
 {
   if(p[0]==0 || (p[0]=='.' && p[1]==0)) {
     d_storage.assign(1, (char)0);
@@ -49,7 +49,7 @@ DNSName::DNSName(const char* p)
     if(!strchr(p, '\\')) {
       unsigned char lenpos=0;
       unsigned char labellen=0;
-      size_t plen=strlen(p);
+      size_t plen=length;
       const char* const pbegin=p, *pend=p+plen;
       d_storage.reserve(plen+1);
       for(auto iter = pbegin; iter != pend; ) {
@@ -76,7 +76,7 @@ DNSName::DNSName(const char* p)
       d_storage.append(1, (char)0);
     }
     else {
-      d_storage=segmentDNSNameRaw(p); 
+      d_storage=segmentDNSNameRaw(p, length);
       if(d_storage.size() > 255) {
         throw std::range_error("name too long");
       }
@@ -126,7 +126,7 @@ void DNSName::packetParser(const char* qpos, int len, int offset, bool uncompres
           throw std::range_error("Invalid label position during decompression ("+std::to_string(newpos)+ " < "+std::to_string(minOffset)+")");
         if (++depth > 100)
           throw std::range_error("Abort label decompression after 100 redirects");
-        packetParser((const char*)opos, len, newpos, true, 0, 0, 0, depth, minOffset);
+        packetParser((const char*)opos, len, newpos, true, nullptr, nullptr, nullptr, depth, minOffset);
       } else
         throw std::range_error("Found a forward reference during label decompression");
       pos++;
@@ -161,16 +161,26 @@ void DNSName::packetParser(const char* qpos, int len, int offset, bool uncompres
 }
 
 std::string DNSName::toString(const std::string& separator, const bool trailing) const
+{
+  std::string ret;
+  toString(ret, separator, trailing);
+  return ret;
+}
+
+void DNSName::toString(std::string& output, const std::string& separator, const bool trailing) const
 {
   if (empty()) {
     throw std::out_of_range("Attempt to print an unset dnsname");
   }
 
-  if(isRoot())
-    return trailing ? separator : "";
+  if (isRoot()) {
+    output += (trailing ? separator : "");
+    return;
+  }
 
-  std::string ret;
-  ret.reserve(d_storage.size());
+  if (output.capacity() < (output.size() + d_storage.size())) {
+    output.reserve(output.size() + d_storage.size());
+  }
 
   {
     // iterate over the raw labels
@@ -178,15 +188,15 @@ std::string DNSName::toString(const std::string& separator, const bool trailing)
     const char* end = p + d_storage.size();
 
     while (p < end && *p) {
-      appendEscapedLabel(ret, p + 1, static_cast<size_t>(*p));
-      ret += separator;
+      appendEscapedLabel(output, p + 1, static_cast<size_t>(*p));
+      output += separator;
       p += *p + 1;
     }
   }
+
   if (!trailing) {
-    ret.resize(ret.size() - separator.size());
+    output.resize(output.size() - separator.size());
   }
-  return ret;
 }
 
 std::string DNSName::toLogString() const
@@ -208,7 +218,9 @@ std::string DNSName::toDNSString() const
 
 std::string DNSName::toDNSStringLC() const
 {
-  return toLower(toDNSString()); // label lengths are always < 'A'
+  auto result = toDNSString();
+  toLowerInPlace(result); // label lengths are always < 'A'
+  return result;
 }
 
 /**
@@ -254,8 +266,9 @@ DNSName DNSName::makeRelative(const DNSName& zone) const
 {
   DNSName ret(*this);
   ret.makeUsRelative(zone);
-  return ret.empty() ? zone : ret; // HACK FIXME400
+  return ret;
 }
+
 void DNSName::makeUsRelative(const DNSName& zone) 
 {
   if (isPartOf(zone)) {
@@ -456,3 +469,25 @@ void DNSName::appendEscapedLabel(std::string& appendTo, const char* orig, size_t
     ++pos;
   }
 }
+
+bool DNSName::has8bitBytes() const
+{
+  const auto& s = d_storage;
+  string::size_type pos = 0;
+  uint8_t length = s.at(pos);
+  while (length > 0) {
+    for (size_t idx = 0; idx < length; idx++) {
+      ++pos;
+      char c = s.at(pos);
+      if(!((c >= 'a' && c <= 'z') ||
+           (c >= 'A' && c <= 'Z') ||
+           (c >= '0' && c <= '9') ||
+           c =='-' || c == '_' || c=='*' || c=='.' || c=='/' || c=='@' || c==' ' || c=='\\' || c==':'))
+        return true;
+    }
+    ++pos;
+    length = s.at(pos);
+  }
+
+  return false;
+}
