@@ -7,6 +7,23 @@
 
 #include "dnsdist-rules.hh"
 
+static DNSQuestion getDQ(const DNSName* providedName = nullptr)
+{
+  static const DNSName qname("powerdns.com.");
+  static const ComboAddress lc("127.0.0.1:53");
+  static const ComboAddress rem("192.0.2.1:42");
+  static struct timespec queryRealTime;
+  static PacketBuffer packet(sizeof(dnsheader));
+
+  uint16_t qtype = QType::A;
+  uint16_t qclass = QClass::IN;
+  auto proto = dnsdist::Protocol::DoUDP;
+  gettime(&queryRealTime, true);
+
+  DNSQuestion dq(providedName ? providedName : &qname, qtype, qclass, &lc, &rem, packet, proto, &queryRealTime);
+  return dq;
+}
+
 BOOST_AUTO_TEST_SUITE(dnsdistluarules_cc)
 
 BOOST_AUTO_TEST_CASE(test_MaxQPSIPRule) {
@@ -23,14 +40,14 @@ BOOST_AUTO_TEST_CASE(test_MaxQPSIPRule) {
   ComboAddress lc("127.0.0.1:53");
   ComboAddress rem("192.0.2.1:42");
   PacketBuffer packet(sizeof(dnsheader));
-  bool isTcp = false;
+  auto proto = dnsdist::Protocol::DoUDP;
   struct timespec queryRealTime;
   gettime(&queryRealTime, true);
   struct timespec expiredTime;
   /* the internal QPS limiter does not use the real time */
   gettime(&expiredTime);
 
-  DNSQuestion dq(&qname, qtype, qclass, &lc, &rem, packet, isTcp, &queryRealTime);
+  DNSQuestion dq(&qname, qtype, qclass, &lc, &rem, packet, proto, &queryRealTime);
 
   for (size_t idx = 0; idx < maxQPS; idx++) {
     /* let's use different source ports, it shouldn't matter */
@@ -101,5 +118,32 @@ BOOST_AUTO_TEST_CASE(test_MaxQPSIPRule) {
   BOOST_CHECK_EQUAL(scanned, 0U);
 }
 
+BOOST_AUTO_TEST_CASE(test_poolOutstandingRule) {
+  auto dq = getDQ();
+
+  ServerPool sp{};
+  auto ds1 = std::make_shared<DownstreamState>(ComboAddress("192.0.2.1:53"));
+  auto ds2 = std::make_shared<DownstreamState>(ComboAddress("192.0.2.2:53"));
+
+  /* increase the outstanding count of both */
+  ds1->outstanding = 400;
+  ds2->outstanding = 30;
+
+  sp.addServer(ds1);
+  sp.addServer(ds2);
+
+  BOOST_CHECK_EQUAL(sp.poolLoad(), 400U + 30U);
+
+  auto localPool = g_pools.getCopy();
+  addServerToPool(localPool, "test", ds1);
+  addServerToPool(localPool, "test", ds2);
+  g_pools.setState(localPool);
+
+  PoolOutstandingRule pOR1("test", 10);
+  BOOST_CHECK_EQUAL(pOR1.matches(&dq), true);
+
+  PoolOutstandingRule pOR2("test", 1000);
+  BOOST_CHECK_EQUAL(pOR2.matches(&dq), false);
+}
 
 BOOST_AUTO_TEST_SUITE_END()
